@@ -1,17 +1,19 @@
 from argparse import (ArgumentParser, _ArgumentGroup, Namespace,
-                      ArgumentDefaultsHelpFormatter)
+                      ArgumentDefaultsHelpFormatter, Action)
 import json
 import os
 import pathlib
 from typing import (Any, Collection, Dict, List,
                     Sequence, Tuple, Union, Optional)
+import warnings
 
 import numpy as np
 from pint import UndefinedUnitError
 import seekpath
 
 from euphonic import (Crystal, DebyeWaller, ForceConstants, QpointFrequencies,
-                      QpointPhononModes, Quantity, ureg)
+                      QpointPhononModes, Spectrum1D, Spectrum1DCollection,
+                      Quantity, ureg)
 import euphonic.util
 Unit = ureg.Unit
 
@@ -31,23 +33,27 @@ def force_constants_from_file(filename: Union[str, os.PathLike]
     ForceConstants
     """
     path = pathlib.Path(filename)
-    if path.suffix == '.hdf5':
-        if (path.parent / 'phonopy.yaml').is_file():
-            return ForceConstants.from_phonopy(path=path.parent,
-                                               summary_name='phonopy.yaml',
-                                               fc_name=path.name)
-        raise ValueError("Phonopy force_constants.hdf5 file "
-                         "must be accompanied by phonopy.yaml")
-    elif path.suffix == '.yaml':
-        # Assume this is a (renamed?) phonopy.yaml file
-        if (path.parent / 'force_constants.hdf5').is_file():
-            fc_name = 'force_constants.hdf5'
-        else:
-            fc_name = 'FORCE_CONSTANTS'
-
-        return ForceConstants.from_phonopy(path=path.parent,
-                                           fc_name=fc_name,
-                                           summary_name=path.name)
+    if path.suffix in ['.hdf5', '.yaml']:
+        phonopy_kwargs = {}
+        phonopy_kwargs['path'] = path.parent
+        if (path.parent / 'BORN').is_file():
+            phonopy_kwargs['born_name'] = 'BORN'
+        # Set summary_name and fc_name depending on input file
+        if path.suffix == '.hdf5':
+            if (path.parent / 'phonopy.yaml').is_file():
+                phonopy_kwargs['summary_name'] = 'phonopy.yaml'
+                phonopy_kwargs['fc_name'] = path.name
+            else:
+                raise ValueError("Phonopy force_constants.hdf5 file "
+                                 "must be accompanied by phonopy.yaml")
+        elif path.suffix == '.yaml':
+            phonopy_kwargs['summary_name'] = path.name
+            # Assume this is a (renamed?) phonopy.yaml file
+            if (path.parent / 'force_constants.hdf5').is_file():
+                phonopy_kwargs['fc_name'] = 'force_constants.hdf5'
+            else:
+                phonopy_kwargs['fc_name'] = 'FORCE_CONSTANTS'
+        return ForceConstants.from_phonopy(**phonopy_kwargs)
     elif path.suffix in ('.castep_bin', '.check'):
         return ForceConstants.from_castep(filename)
     elif path.suffix == '.json':
@@ -195,29 +201,30 @@ def _get_q_distance(length_unit_string: str, q_distance: float) -> Quantity:
     return q_distance * recip_length_units
 
 
-def _get_energy_bins_and_units(
-        energy_unit_string: str, modes: QpointPhononModes,
+def _get_energy_bins(
+        modes: QpointPhononModes,
         n_ebins: int, emin: Optional[float] = None,
         emax: Optional[float] = None,
-        headroom: float = 1.05) -> Tuple[Quantity, Unit]:
-
-    try:
-        energy_unit = ureg(energy_unit_string)
-    except UndefinedUnitError:
-        raise ValueError("Energy unit not known. Euphonic uses Pint for units."
-                         " Try 'eV' or 'hartree'. Metric prefixes are also "
-                         "allowed, e.g 'meV' or 'fJ'.")
-
+        headroom: float = 1.05) -> Quantity:
+    """
+    Gets recommeded energy bins, in same units as modes.frequencies.
+    emin and emax are assumed to be in the same units as
+    modes.frequencies, if not provided the min/max values of
+    modes.frequencies are used to find the bin limits
+    """
     if emin is None:
-        emin = min(np.min(modes.frequencies.magnitude), 0.)
+        # Subtract small amount from min frequency - otherwise due to unit
+        # conversions binning of this frequency can vary with different
+        # architectures/lib versions, making it difficult to test
+        emin_room = 1e-5*ureg('meV').to(modes.frequencies.units).magnitude
+        emin = min(np.min(modes.frequencies.magnitude - emin_room), 0.)
     if emax is None:
         emax = np.max(modes.frequencies.magnitude) * headroom
     if emin >= emax:
         raise ValueError("Maximum energy should be greater than minimum. "
                          "Check --e-min and --e-max arguments.")
-    ebins = np.linspace(emin, emax, n_ebins) * energy_unit
-
-    return ebins, energy_unit
+    ebins = np.linspace(emin, emax, n_ebins) * modes.frequencies.units
+    return ebins
 
 
 def _get_tick_labels(bandpath: dict) -> List[Tuple[int, str]]:
@@ -368,10 +375,47 @@ def _get_debye_waller(temperature: Quantity,
     return dw_phonons.calculate_debye_waller(temperature)
 
 
+def _get_pdos_weighting(cl_arg_weighting: str) -> str:
+    """
+    Convert CL --weighting to weighting for calculate_pdos
+    e.g. --weighting coherent-dos to weighting=coherent
+    """
+    if cl_arg_weighting == 'dos':
+        pdos_weighting = None
+    else:
+        idx = cl_arg_weighting.rfind('-')
+        if idx == -1:
+            raise ValueError('Unexpected weighting {cl_arg_weighting}')
+        pdos_weighting = cl_arg_weighting[:idx]
+    return pdos_weighting
+
+
+def _arrange_pdos_groups(pdos: Spectrum1DCollection,
+                         cl_arg_pdos: Sequence[str]
+                         ) -> Union[Spectrum1D, Spectrum1DCollection]:
+    """
+    Convert PDOS returned by calculate_pdos to PDOS/DOS
+    wanted as CL output according to --pdos
+    """
+    dos = pdos.sum()
+    if cl_arg_pdos is not None:
+        # Only label total DOS if there are other lines on the plot
+        dos.metadata['label'] = 'Total'
+        pdos = pdos.group_by('species')
+        for line_metadata in pdos.metadata['line_data']:
+            line_metadata['label'] = line_metadata['species']
+        if len(cl_arg_pdos) > 0:
+            pdos = pdos.select(species=cl_arg_pdos)
+            dos = pdos
+        else:
+            dos = Spectrum1DCollection.from_spectra([dos] + [*pdos])
+    return dos
+
+
 def _calc_modes_kwargs(args: Namespace) -> Dict[str, Any]:
     """Collect arguments that can be passed to calculate_qpoint_phonon_modes()
     """
-    return dict(asr=args.asr, eta_scale=args.eta_scale,
+    return dict(asr=args.asr, dipole_parameter=args.dipole_parameter,
                 use_c=args.use_c, n_threads=args.n_threads)
 
 
@@ -383,15 +427,36 @@ def _get_cli_parser(features: Collection[str] = {}
     Args:
         sections:
             collection (e.g. set, list) of str for known argument groups.
-            Known keys: read-fc, read-modes, weights, powder, mp-grid,
-                plotting, ebins, adaptive-broadening, q-e, map, btol
+            Known keys: read-fc, read-modes, ins-weighting, pdos-weighting,
+            powder, mp-grid, plotting, ebins, adaptive-broadening, q-e,
+            map, btol, dipole-parameter-optimisation
 
     Returns:
         Parser and a dict of parser subsections. This allows their help strings
         to be customised and specialised options to be added.
 
     """
-    _spectrum_choices = ('dos', 'coherent')
+    def deprecation_text(deprecated_arg: str, new_arg: str):
+        return (f'--{deprecated_arg} is deprecated, '
+                f'please use --{new_arg} instead')
+    def deprecated_arg(recommended_arg: str):
+        class DeprecatedArgAction(Action):
+            def __call__(self, parser, args, values, option_string=None):
+                # Need to filter to raise warnings from CL tools
+                # Warnings not in __main__ are ignored by default
+                with warnings.catch_warnings():
+                    warnings.filterwarnings(
+                        'default', category=DeprecationWarning,
+                        module=__name__)
+                    warnings.warn(
+                        deprecation_text(self.dest, recommended_arg),
+                        DeprecationWarning)
+                    setattr(args, recommended_arg, values)
+        return DeprecatedArgAction
+
+    _pdos_choices = ('coherent-dos', 'incoherent-dos',
+                     'coherent-plus-incoherent-dos')
+    _ins_choices = ('coherent',)
 
     parser = ArgumentParser(
         formatter_class=ArgumentDefaultsHelpFormatter)
@@ -438,13 +503,15 @@ def _get_cli_parser(features: Collection[str] = {}
                   'data: "realspace" applies the correction to the force '
                   'constant matrix in real space. "reciprocal" applies '
                   'the correction to the dynamical matrix at each q-point.'))
-        sections['interpolation'].add_argument(
-            '--eta-scale', type=float, default=1.0, dest='eta_scale',
-            help=('Set the cutoff in real/reciprocal space for the dipole '
-                  'Ewald sum; higher values use more reciprocal terms. If '
-                  'tuned correctly this can result in performance '
-                  'improvements. See euphonic-optimise-eta program for help '
-                  'on choosing a good ETA_SCALE.'))
+        if not 'dipole-parameter-optimisation' in features:
+            sections['interpolation'].add_argument(
+                '--dipole-parameter', type=float, default=1.0,
+                dest='dipole_parameter',
+                help=('Set the cutoff in real/reciprocal space for the dipole '
+                      'Ewald sum; higher values use more reciprocal terms. If '
+                      'tuned correctly this can result in performance '
+                      'improvements. See euphonic-optimise-dipole-parameter '
+                      'program for help on choosing a good DIPOLE_PARAMETER.'))
 
         use_c = sections['performance'].add_mutually_exclusive_group()
         use_c.add_argument(
@@ -460,19 +527,60 @@ def _get_cli_parser(features: Collection[str] = {}
             help=('Number of parallel processes for computing phonon modes. '
                   '(Only applies when using C extension.)'))
 
-    if 'weights' in features:
-        sections['property'].add_argument(
-            '--weights', '-w', default='dos', choices=_spectrum_choices,
-            help=('Spectral weights to plot: phonon DOS or '
-                  'coherent inelastic neutron scattering.'))
+    pdos_desc = ('coherent neutron-weighted DOS, incoherent '
+                 'neutron-weighted DOS or total (coherent + incoherent) '
+                 'neutron-weighted DOS')
+    ins_desc = ('coherent inelastic neutron scattering')
+    if 'pdos-weighting' in features:
+        # Currently do not support multiple PDOS for 2D plots
+        if not 'q-e' in features:
+            sections['property'].add_argument(
+                '--pdos', type=str, action='store', nargs='*',
+                help=('Plot PDOS. With --pdos, per-species PDOS will be plotted '
+                      'alongside total DOS. A subset of species can also be '
+                      'selected by adding more arguments e.g. --pdos Si O'))
+        else:
+            sections['property'].add_argument(
+                '--pdos', type=str, action='store', nargs=1,
+                help=('Plot PDOS for a specific species e.g. --pdos Si'))
 
+    if {'pdos-weighting', 'ins-weighting'}.intersection(
+            features):
+        # We can plot plain DOS with all CL tools
+        _weighting_choices = ('dos',)
+        desc = 'DOS'
+        if 'pdos-weighting' in features:
+            _weighting_choices += _pdos_choices
+            desc += (', coherent neutron-weighted DOS, incoherent '
+                     'neutron-weighted DOS or total (coherent + incoherent) '
+                     'neutron-weighted DOS')
+        # We may have both 'pdos' and 'ins' so use separate if
+        if 'ins-weighting' in features:
+            _weighting_choices += _ins_choices
+            desc += ', coherent inelastic neutron scattering'
+            # --weights was deprecated before dos-weighting was added so
+            # keep in this section
+            sections['property'].add_argument(
+                '--weights', default='dos', choices=_weighting_choices,
+                action=deprecated_arg('weighting'),
+                help=deprecation_text('weights', 'weighting'))
+            sections['property'].add_argument(
+                '--weighting', '-w', default='dos', choices=_weighting_choices,
+                help=(f'Spectral weighting to plot: {desc}'))
+        else:
+            sections['property'].add_argument(
+                '--weighting', '-w', default='dos', choices=_weighting_choices,
+                help=f'Type of DOS to plot: {desc}')
+
+    if 'ins-weighting' in features:
         sections['property'].add_argument(
             '--temperature', type=float, default=None,
             help=('Temperature in K; enable Debye-Waller factor calculation. '
-                  '(Only applicable when --weights=coherent).'))
+                  '(Only applicable when --weighting=coherent).'))
 
-    # 'weights' implies 'mp-grid' as well because mesh is needed for DW factor
-    if {'weights', 'mp-grid'}.intersection(features):
+    # 'ins-weighting' implies 'mp-grid' as well because mesh is needed for DW
+    # factor
+    if {'ins-weighting', 'mp-grid'}.intersection(features):
         grid_spec = sections['q'].add_mutually_exclusive_group()
 
         grid_spec.add_argument(
@@ -605,4 +713,32 @@ def _get_cli_parser(features: Collection[str] = {}
                   'subplots. This is specified as a multiple of the median '
                   'distance between q-points.'))
 
+    if 'dipole-parameter-optimisation' in features:
+        parser.add_argument(
+            '-n',
+            default=500,
+            type=int,
+            help=('The number of times to loop over q-points. A higher '
+                  'value will get a more reliable timing, but will take '
+                  'longer')
+        )
+        parser.add_argument(
+            '--dipole-parameter-min', '--min',
+            default=0.25,
+            type=float,
+            help='The minimum value of dipole_parameter to test'
+        )
+        parser.add_argument(
+            '--dipole-parameter-max', '--max',
+            default=1.5,
+            type=float,
+            help='The maximum value of dipole_parameter to test'
+        )
+        parser.add_argument(
+            '--dipole-parameter-step', '--step',
+            default=0.25,
+            type=float,
+            help='The difference between each dipole_parameter to test'
+        )
+
     return parser, sections
