@@ -6,8 +6,6 @@ from typing import Optional, Tuple, Union
 from multiprocessing import cpu_count
 
 import numpy as np
-import scipy
-from scipy.linalg.lapack import zheev
 from scipy.special import erfc
 from threadpoolctl import threadpool_limits
 
@@ -112,6 +110,11 @@ class ForceConstants:
             'hartree/bohr**2').to(
                 self.force_constants_unit)
 
+    @force_constants.setter
+    def force_constants(self, value):
+        self.force_constants_unit = str(value.units)
+        self._force_constants = value.to('hartree/bohr**2').magnitude
+
     @property
     def born(self):
         if self._born is not None:
@@ -119,6 +122,11 @@ class ForceConstants:
         else:
             return None
 
+    @born.setter
+    def born(self, value):
+        self.born_unit = str(value.units)
+        self._born = value.to('e').magnitude
+
     @property
     def dielectric(self):
         if self._dielectric is not None:
@@ -127,6 +135,11 @@ class ForceConstants:
         else:
             return None
 
+    @dielectric.setter
+    def dielectric(self, value):
+        self.dielectric_unit = str(value.units)
+        self._dielectric = value.to('e**2/(bohr*hartree)').magnitude
+
     def __setattr__(self, name, value):
         _check_unit_conversion(self, name, value,
                                ['force_constants_unit', 'born_unit',
@@ -139,12 +152,14 @@ class ForceConstants:
             weights: Optional[np.ndarray] = None,
             asr: Optional[str] = None,
             dipole: bool = True,
+            dipole_parameter: float = 1.0,
             eta_scale: float = 1.0,
             splitting: bool = True,
             insert_gamma: bool = False,
             reduce_qpts: bool = True,
             use_c: Optional[bool] = None,
             n_threads: Optional[int] = None,
+            return_mode_gradients: bool = False,
             return_mode_widths: bool = False
             ) -> Union[QpointPhononModes,
                        Tuple[QpointPhononModes, Quantity]]:
@@ -168,12 +183,16 @@ class ForceConstants:
             Whether to calculate the dipole tail correction to the dynamical
             matrix at each q-point using the Ewald sum, if the Born
             charges and dielectric permitivitty tensor are present.
-        eta_scale
+        dipole_parameter
             Changes the cutoff in real/reciprocal space for the dipole
             Ewald sum. A higher value uses more reciprocal terms. If tuned
             correctly this can result in performance improvements. See
-            euphonic-optimise-eta program for help on choosing a good
-            eta_scale.
+            euphonic-optimise-dipole-parameter program for help on choosing
+            a good dipole_parameter.
+        eta_scale
+
+            .. deprecated:: 0.6.0
+            Please use dipole_parameter instead
         splitting
             Whether to calculate the LO-TO splitting at the gamma
             points. Only applied if dipole is True and the Born charges
@@ -204,10 +223,21 @@ class ForceConstants:
             to determine number of threads, if this is not set then
             the value returned from multiprocessing.cpu_count() will be
             used
+        return_mode_gradients
+            Whether to also return the vector mode gradients (in
+            Cartesian coordinates). These can be converted to mode
+            widths and used in adaptive broadening for DOS. For details
+            on how these are calculated see the Notes section
         return_mode_widths
-            Whether to also return the mode widths. The mode widths can
-            be used in adaptive broadening for DOS. For details on how
-            these are calculated see the Notes section
+
+            .. deprecated:: 0.5.2
+            The mode widths as calculated were only applicable for
+            adaptive broadening of DOS, this argument will be removed
+            in favour of the more flexible return_mode_gradients,
+            which will allow the calculation of direction-specific
+            mode widths in the future, for example. The mode widths
+            can still be obtained from the mode gradients using
+            euphonic.util.mode_gradients_to_widths
 
         Returns
         -------
@@ -217,9 +247,10 @@ class ForceConstants:
             there is LO-TO splitting, and insert_gamma=True, the number
             of input q-points may not be the same as in the output
             object
-        mode_widths
-            Optional shape (n_qpts, n_branches) float Quantity. Is only
-            returned if return_mode_widths is true
+        mode_gradients
+            Optional shape (n_qpts, n_branches, 3) float Quantity,
+            the vector mode gradients dw/dQ in Cartesian coordinates.
+            Is only returned if return_mode_gradients is true
 
 
         Raises
@@ -275,16 +306,18 @@ class ForceConstants:
         .. [1] M. T. Dove, Introduction to Lattice Dynamics, Cambridge University Press, Cambridge, 1993, 83-87
         .. [2] X. Gonze, K. C. Charlier, D. C. Allan, M. P. Teter, Phys. Rev. B, 1994, 50, 13035-13038
 
-        **Mode Widths Calculation**
+        **Mode Gradients Calculation**
 
-        The mode widths are used in the adaptive broadening scheme [3]_
-        when computing a DOS - broadening each mode contribution
-        individually according to its mode width. The mode widths at
-        each Q are calculated as the same time as the phonon
-        frequencies and eigenvectors as follows.
+        The mode gradients can be used to calculate mode widths to be used
+        in the adaptive broadening scheme [3]_ when computing a DOS - this
+        broadens each mode contribution individually according to its mode
+        width. The mode widths are proportional to the mode gradients and
+        can be estimated using ``euphonic.util.mode_gradients_to_widths``
+
+        The mode gradients :math:`\\frac{d\\omega_{q\\nu}}{dQ}` at each Q
+        are calculated as the same time as the phonon frequencies and
+        eigenvectors as follows.
 
-        The mode widths are proportional to the gradient of the
-        dispersion, :math:`\\frac{d\\omega_{q\\nu}}{dQ}`.
         Firstly, the eigenvalue equation above can be written in matrix
         form as:
 
@@ -335,14 +368,15 @@ class ForceConstants:
 
         .. [3] J. R. Yates, X. Wang, D. Vanderbilt and I. Souza, Phys. Rev. B, 2007, 75, 195121
         """
-        qpts, weights, freqs, evecs, widths = self._calculate_phonons_at_qpts(
-            qpts, weights, asr, dipole, eta_scale, splitting, insert_gamma,
-            reduce_qpts, use_c, n_threads, return_mode_widths,
+        qpts, weights, freqs, evecs, grads = self._calculate_phonons_at_qpts(
+            qpts, weights, asr, dipole, dipole_parameter, eta_scale,
+            splitting, insert_gamma, reduce_qpts, use_c, n_threads,
+            return_mode_gradients, return_mode_widths,
             return_eigenvectors=True)
         qpt_ph_modes = QpointPhononModes(
             self.crystal, qpts, freqs, evecs, weights=weights)
-        if return_mode_widths:
-            return qpt_ph_modes, widths
+        if return_mode_gradients or return_mode_widths:
+            return qpt_ph_modes, grads
         else:
             return qpt_ph_modes
 
@@ -352,12 +386,14 @@ class ForceConstants:
             weights: Optional[np.ndarray] = None,
             asr: Optional[str] = None,
             dipole: bool = True,
+            dipole_parameter: float = 1.0,
             eta_scale: float = 1.0,
             splitting: bool = True,
             insert_gamma: bool = False,
             reduce_qpts: bool = True,
             use_c: Optional[bool] = None,
             n_threads: Optional[int] = None,
+            return_mode_gradients: bool = False,
             return_mode_widths: bool = False,
             ) -> Union[QpointFrequencies,
                        Tuple[QpointFrequencies, Quantity]]:
@@ -366,14 +402,15 @@ class ForceConstants:
         q-points. See ForceConstants.calculate_qpoint_phonon_modes for
         argument and algorithm details
         """
-        qpts, weights, freqs, _, widths = self._calculate_phonons_at_qpts(
-            qpts, weights, asr, dipole, eta_scale, splitting, insert_gamma,
-            reduce_qpts, use_c, n_threads, return_mode_widths,
+        qpts, weights, freqs, _, grads = self._calculate_phonons_at_qpts(
+            qpts, weights, asr, dipole, dipole_parameter, eta_scale,
+            splitting, insert_gamma, reduce_qpts, use_c, n_threads,
+            return_mode_gradients, return_mode_widths,
             return_eigenvectors=False)
         qpt_freqs = QpointFrequencies(
             self.crystal, qpts, freqs, weights=weights)
-        if return_mode_widths:
-            return qpt_freqs, widths
+        if return_mode_gradients or return_mode_widths:
+            return qpt_freqs, grads
         else:
             return qpt_freqs
 
@@ -383,16 +420,36 @@ class ForceConstants:
             weights: np.ndarray,
             asr: str,
             dipole: bool,
+            dipole_parameter: float,
             eta_scale: float,
             splitting: bool,
             insert_gamma: bool,
             reduce_qpts: bool,
             use_c: Optional[bool],
             n_threads: Optional[int],
+            return_mode_gradients: bool,
             return_mode_widths: bool,
             return_eigenvectors: bool) -> Union[
                 Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray],
                 Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray, np.ndarray]]:
+        if return_mode_widths:
+            warnings.warn(
+                'return_mode_widths has been deprecated and will be removed '
+                'in a future release. Please instead use a combination of '
+                'return_mode_gradients and euphonic.util.'
+                'mode_gradients_to_widths, e.g.:\n'
+                'phon, mode_gradients = force_constants.calculate_qpoint_phonon_modes('
+                '*args, **kwargs, return_mode_gradients=True)\n'
+                'mode_widths = euphonic.util.mode_gradients_to_widths(mode_gradients, '
+                'phon.crystal.cell_vectors)',
+                category=DeprecationWarning, stacklevel=3)
+            return_mode_gradients = True
+        if eta_scale != 1.0:
+            warnings.warn(
+                'eta_scale has been deprecated and will be removed '
+                'in a future release. Please use dipole_parameter instead ',
+                category=DeprecationWarning, stacklevel=3)
+            dipole_parameter = eta_scale
         # Check weights is of appropriate type and shape, to avoid doing all
         # the interpolation only for it to fail creating QpointPhononModes
         _check_constructor_inputs(
@@ -489,7 +546,7 @@ class ForceConstants:
                 sc_origins[:, i], return_inverse=True)
             unique_cell_origins[i], unique_cell_i[:, i] = np.unique(
                 self.cell_origins[:, i], return_inverse=True)
-        if return_mode_widths:
+        if return_mode_gradients:
             cell_origins_cart = np.einsum('ij,jk->ik', self.cell_origins,
                                           self.crystal._cell_vectors)
             # Append 0. to sc_origins_cart, so that when being indexed by
@@ -511,9 +568,9 @@ class ForceConstants:
         dyn_mat_weighting = 1/np.sqrt(masses*np.transpose(masses))
 
         # Initialise dipole correction calc to FC matrix if required
-        if dipole and (not hasattr(self, 'eta_scale') or
-                       eta_scale != self._eta_scale):
-            self._dipole_correction_init(eta_scale)
+        if dipole and (not hasattr(self, '_dipole_parameter') or
+                       dipole_parameter != self._dipole_parameter):
+            self._dipole_correction_init(dipole_parameter)
 
         if asr == 'realspace':
             if not hasattr(self, '_force_constants_asr'):
@@ -554,10 +611,11 @@ class ForceConstants:
             reigenvecs = np.zeros(
                 (0, 3*n_atoms, n_atoms, 3), dtype=np.complex128)
 
-        if return_mode_widths:
-            rmode_gradients = np.zeros((n_rqpts, 3*n_atoms), dtype=np.float64)
+        if return_mode_gradients:
+            rmode_gradients = np.zeros((n_rqpts, 3*n_atoms, 3),
+                                       dtype=np.complex128)
         else:
-            rmode_gradients = np.zeros((0, 3*n_atoms), dtype=np.float64)
+            rmode_gradients = np.zeros((0, 3*n_atoms, 3), dtype=np.complex128)
 
         euphonic_path = os.path.dirname(euphonic.__file__)
         cext_err_msg = (f'Euphonic\'s C extension couldn\'t be imported '
@@ -591,6 +649,12 @@ class ForceConstants:
             cell_vectors = self.crystal._cell_vectors
             recip_vectors = self.crystal.reciprocal_cell().to(
                 '1/bohr').magnitude
+            # Get conj transpose - the dynamical matrix calculated in C
+            # is passed to Fortran libs so uses Fortran ordering, make
+            # sure reciprocal correction matches this. This only makes
+            # a very small difference and can only be seen in near-flat
+            # mode gradients, but should be corrected anyway
+            recip_asr_correction =  recip_asr_correction.conj().T
             (cell_vectors, recip_vectors, reduced_qpts, split_idx, q_dirs,
              fc_img_weighted, sc_origins, recip_asr_correction,
              dyn_mat_weighting, rfreqs, reigenvecs, rmode_gradients,
@@ -611,8 +675,7 @@ class ForceConstants:
                     split_idx, q_dirs, fc_img_weighted, sc_origins,
                     recip_asr_correction, dyn_mat_weighting, dipole,
                     reciprocal_asr, splitting, rfreqs, reigenvecs,
-                    rmode_gradients, all_origins_cart, n_threads,
-                    scipy.__path__[0])
+                    rmode_gradients, all_origins_cart, n_threads)
         else:
             q_independent_args = (
                 reduced_qpts, split_idx, q_dirs, fc_img_weighted,
@@ -624,7 +687,7 @@ class ForceConstants:
                         q, q_independent_args)
                 if return_eigenvectors:
                     reigenvecs[q] = evecs
-                if return_mode_widths:
+                if return_mode_gradients:
                     rmode_gradients[q] = grads
 
         freqs = rfreqs[qpts_i]*ureg('hartree').to('meV')
@@ -632,13 +695,31 @@ class ForceConstants:
             eigenvectors = reigenvecs[qpts_i]
         else:
             eigenvectors = None
+
+        mode_gradients = None
+        if return_mode_gradients:
+            max_real = np.amax(np.absolute(rmode_gradients.real))
+            idx = np.where(
+                rmode_gradients.imag/max_real > 1e-10)
+            n_idx = len(idx[0])
+            if n_idx > 0:
+                n_print = n_idx if n_idx < 5 else 5
+                warnings.warn(
+                    f'Unexpected values for mode gradients at {n_idx}/{rmode_gradients.size} '
+                    f'indices {[x for x in zip(*idx)][:n_print]}..., '
+                    f'expected near-zero imaginary elements, got values of '
+                    f'{rmode_gradients.imag[idx][:n_print]}..., compared to a '
+                    f'max real value of {max_real}. Data may have been lost '
+                    f'when casting to real mode gradients',
+                    stacklevel=3)
+            mode_gradients = rmode_gradients.real[qpts_i]*ureg(
+                'hartree*bohr').to(
+                    f'meV*{str(self.crystal.cell_vectors.units)}')
         if return_mode_widths:
-            mode_widths = mode_gradients_to_widths(
-                rmode_gradients[qpts_i]*ureg('hartree*bohr'),
-                self.crystal.cell_vectors).to('meV')
-        else:
-            mode_widths = None
-        return qpts, weights, freqs, eigenvectors, mode_widths
+            mode_gradients = mode_gradients_to_widths(
+                mode_gradients,
+                self.crystal.cell_vectors)
+        return qpts, weights, freqs, eigenvectors, mode_gradients
 
     def _calculate_phonons_at_q(self, q, args):
         """
@@ -692,14 +773,11 @@ class ForceConstants:
             n_modes = len(evecs)
             dmat_grad *= dyn_mat_weighting[..., np.newaxis]
             evecs_sq_view = np.reshape(evecs, (n_modes, n_modes))
-            grad_tmps = np.einsum('ij,ik,jkl->il', np.conj(evecs_sq_view),
-                                                   evecs_sq_view,
-                                                   dmat_grad)
-            mode_gradients = np.zeros(n_modes)
-            for n, grad_tmp in enumerate(grad_tmps):
-                mode_gradients[n] = 0.5*np.linalg.norm(grad_tmp)/evals[n]
-
-            return evals, evecs, mode_gradients
+            mode_grads_xyz = np.einsum(
+                'ij,ik,jkl->il', np.conj(evecs_sq_view),
+                                 evecs_sq_view,
+                                 dmat_grad)/(2*evals[:, np.newaxis])
+            return evals, evecs, mode_grads_xyz
         else:
             return evals, evecs, None
 
@@ -734,16 +812,19 @@ class ForceConstants:
             direction. A list of lists rather than a Numpy array is used
             as the 3 lists are independent and their size is not known
             beforehand
-        unique_sc_i : (cell_origins, 3) int ndarray
+        unique_cell_i : (n_cells_in_sc, 3) int ndarray
             The indices needed to reconstruct cell_origins from the
             unique values in unique_cell_origins
+        all_origins_cart : (n_cells_in_sc, n_atoms, n_atoms, n_images, 3)
+            or (0, 3) float ndarray
+            The Cartesian coordinate of each atom in each supercell
+            image. Is only used if calculating dynamical matrix gradients
 
         Returns
         -------
         dyn_mat : (3*n_atoms, 3*n_atoms) complex ndarray
             The non mass weighted dynamical matrix at q
         """
-
         sc_image_i = self._sc_image_i
 
         # Cumulant method: for each ij ion-ion displacement sum phases
@@ -778,7 +859,7 @@ class ForceConstants:
         else:
             return dyn_mat, None
 
-    def _dipole_correction_init(self, eta_scale=1.0):
+    def _dipole_correction_init(self, dipole_parameter=1.0):
         """
         Calculate the q-independent parts of the long range correction
         to the dynamical matrix for efficiency. The method used is based
@@ -787,7 +868,7 @@ class ForceConstants:
 
         Parameters
         ----------
-        eta_scale : float, optional
+        dipole_parameter : float, optional
             Changes the cutoff in real/reciprocal space for the dipole
             Ewald sum. A higher value uses more reciprocal terms
         """
@@ -801,13 +882,14 @@ class ForceConstants:
         inv_dielectric = np.linalg.inv(dielectric)
         sqrt_pi = math.sqrt(math.pi)
 
-        # Calculate real/recip weighting
+        # Make guess for balance between real/recip terms
         abc_mag = np.linalg.norm(cell_vectors, axis=1)
         mean_abc_mag = np.prod(abc_mag)**(1.0/3)
-        eta = (sqrt_pi/mean_abc_mag)*n_atoms**(1.0/6)
-        # Use eta = lambda * |permittivity|**(1/6)
-        eta = eta*np.power(np.linalg.det(dielectric), 1.0/6)*eta_scale
-        eta_2 = eta**2
+        # Using upper_lambda varname as lambda is a reserved keyword
+        upper_lambda = (sqrt_pi/mean_abc_mag*n_atoms**(1.0/6)
+                        *np.linalg.det(dielectric)**(1.0/6))
+        upper_lambda *= dipole_parameter
+        lambda_2 = upper_lambda**2
 
         # Set limits and tolerances
         max_shells = 50
@@ -837,13 +919,13 @@ class ForceConstants:
                     rij_e = atom_r_e[i] - atom_r_e[j]
                     diffs = rij_cart - cells_cart
                     deltas = rij_e - cells_e
-                    norms_2 = np.einsum('ij,ij->i', deltas, diffs)*eta_2
+                    norms_2 = np.einsum('ij,ij->i', deltas, diffs)*lambda_2
                     norms = np.sqrt(norms_2)
 
                     # Calculate H_ab
                     exp_term = 2*np.exp(-norms_2)/(sqrt_pi*norms_2)
                     erfc_term = erfc(norms)/(norms*norms_2)
-                    f1 = eta_2*(3*erfc_term/norms_2 + exp_term*(3/norms_2 + 2))
+                    f1 = lambda_2*(3*erfc_term/norms_2 + exp_term*(3/norms_2 + 2))
                     f2 = erfc_term + exp_term
                     deltas_ab = np.einsum('ij,ik->ijk', deltas, deltas)
                     H_ab_tmp[:, idx + j] = (
@@ -860,7 +942,7 @@ class ForceConstants:
                 break
         # Use compact H_ab to fill in upper triangular of realspace term
         real_q0[np.triu_indices(n_atoms)] = np.sum(H_ab, axis=0)
-        real_q0 *= eta**3/math.sqrt(np.linalg.det(dielectric))
+        real_q0 *= upper_lambda**3/math.sqrt(np.linalg.det(dielectric))
 
         # Calculate the q=0 reciprocal term
         recip_q0 = np.zeros((n_atoms, n_atoms, 3, 3), dtype=np.complex128)
@@ -874,7 +956,8 @@ class ForceConstants:
             gvec_dot_r = np.einsum('ij,kj->ik', gvecs, atom_r)
             gvec_phases_tmp = np.exp(2j*math.pi*gvec_dot_r)
             gvecs_ab = np.einsum('ij,ik->ijk', gvecs_cart_tmp, gvecs_cart_tmp)
-            k_len_2 = np.einsum('ijk,jk->i', gvecs_ab, dielectric)/(4*eta_2)
+            k_len_2 = np.einsum(
+                'ijk,jk->i', gvecs_ab, dielectric)/(4*lambda_2)
             recip_exp = np.exp(-k_len_2)/k_len_2
             recip_q0_tmp = np.zeros((n_atoms, n_atoms, 3, 3),
                                     dtype=np.complex128)
@@ -894,7 +977,7 @@ class ForceConstants:
             else:
                 break
         vol = self.crystal._cell_volume()
-        recip_q0 *= math.pi/(vol*eta_2)
+        recip_q0 *= math.pi/(vol*lambda_2)
 
         # Fill in remaining entries by symmetry
         for i in range(1, n_atoms):
@@ -915,8 +998,8 @@ class ForceConstants:
             # Symmetrise 3x3
             dipole_q0[i] = 0.5*(dipole_q0[i] + np.transpose(dipole_q0[i]))
 
-        self._eta_scale = eta_scale
-        self._eta = eta
+        self._dipole_parameter = dipole_parameter
+        self._lambda = upper_lambda
         self._H_ab = H_ab
         self._cells = cells
         self._gvecs_cart = gvecs_cart
@@ -944,8 +1027,8 @@ class ForceConstants:
         atom_r = self.crystal.atom_r
         born = self._born
         dielectric = self._dielectric
-        eta = self._eta
-        eta_2 = eta**2
+        upper_lambda = self._lambda
+        lambda_2 = upper_lambda**2
         H_ab = self._H_ab
         cells = self._cells
         q_norm = q - np.rint(q)  # Normalised q-pt
@@ -966,7 +1049,7 @@ class ForceConstants:
         real_dipole_tmp = np.einsum('i,ijkl->jkl', real_phases, H_ab)
         idx_u = np.triu_indices(n_atoms)
         real_dipole[idx_u] = real_dipole_tmp
-        real_dipole *= eta**3/math.sqrt(np.linalg.det(dielectric))
+        real_dipole *= upper_lambda**3/math.sqrt(np.linalg.det(dielectric))
 
         # Calculate reciprocal term
         recip_dipole = np.zeros((n_atoms, n_atoms, 3, 3), dtype=np.complex128)
@@ -977,7 +1060,7 @@ class ForceConstants:
         # Calculate k-vector symmetric matrix
         kvecs = gvecs_cart + q_cart
         kvecs_ab = np.einsum('ij,ik->ijk', kvecs, kvecs)
-        k_len_2 = np.einsum('ijk,jk->i', kvecs_ab, dielectric)/(4*eta_2)
+        k_len_2 = np.einsum('ijk,jk->i', kvecs_ab, dielectric)/(4*lambda_2)
         recip_exp = np.einsum('ijk,i->ijk', kvecs_ab, np.exp(-k_len_2)/k_len_2)
         for i in range(n_atoms):
                 phase_exp = ((gvec_phases[:, i, None]*q_phases[i])
@@ -985,7 +1068,7 @@ class ForceConstants:
                 recip_dipole[i, i:] = np.einsum(
                     'ikl,ij->jkl', recip_exp, phase_exp)
         cell_volume = self.crystal._cell_volume()
-        recip_dipole *= math.pi/(cell_volume*eta_2)
+        recip_dipole *= math.pi/(cell_volume*lambda_2)
 
         # Fill in remaining entries by symmetry
         # Mask so we don't count diagonal twice
@@ -1146,7 +1229,7 @@ class ForceConstants:
             warnings.warn((
                 '\nError correcting for acoustic sum rule, could not '
                 'find 3 acoustic modes.\nReturning uncorrected FC '
-                'matrix'), stacklevel=2)
+                'matrix'), stacklevel=3)
             return self._force_constants
 
         # Correct fc matrix - set acoustic modes to almost zero
@@ -1189,7 +1272,7 @@ class ForceConstants:
         except Exception:
             warnings.warn(('\nError correcting for acoustic sum rule, '
                            'could not find 3 acoustic modes.\nNot '
-                           'correcting dynamical matrix'), stacklevel=2)
+                           'correcting dynamical matrix'), stacklevel=3)
             return np.array([], dtype=np.complex128)
 
         n_atoms = self.crystal.n_atoms
